@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2024	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -27,8 +27,11 @@
 #include "utils/memutils.h"
 #include "utils/elog.h"
 #include "pool_config.h"
-#include "context/pool_session_context.h"
 #include "protocol/pool_proto_modules.h"
+#include "protocol/pool_process_query.h"
+#include "protocol/pool_connection_pool.h"
+#include "protocol/pool_pg_utils.h"
+#include "context/pool_session_context.h"
 
 static POOL_SESSION_CONTEXT session_context_d;
 static POOL_SESSION_CONTEXT * session_context = NULL;
@@ -36,6 +39,8 @@ static void GetTranIsolationErrorCb(void *arg);
 static void init_sent_message_list(void);
 static POOL_PENDING_MESSAGE * copy_pending_message(POOL_PENDING_MESSAGE * messag);
 static void dump_sent_message(char *caller, POOL_SENT_MESSAGE * m);
+static void dml_adaptive_init(void);
+static void dml_adaptive_destroy(void);
 
 #ifdef PENDING_MESSAGE_DEBUG
 static int	Elevel = LOG;
@@ -97,7 +102,7 @@ pool_init_session_context(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * bac
 	}
 	else
 	{
-		node_id = SL_MODE ? PRIMARY_NODE_ID : MASTER_NODE_ID;
+		node_id = SL_MODE ? PRIMARY_NODE_ID : MAIN_NODE_ID;
 	}
 
 	session_context->load_balance_node_id = node_id;
@@ -136,6 +141,12 @@ pool_init_session_context(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * bac
 	/* Unset suspend reading from frontend flag */
 	pool_unset_suspend_reading_from_frontend();
 
+	/* Initialize where to send map for PREPARE statements */
+#ifdef NOT_USED
+	memset(&session_context->prep_where, 0, sizeof(session_context->prep_where));
+	session_context->prep_where.nelem = POOL_MAX_PREPARED_STATEMENTS;
+#endif							/* NOT_USED */
+
 	/*
 	 * Reset flag to indicate difference in number of affected tuples in
 	 * UPDATE/DELETE.
@@ -156,10 +167,19 @@ pool_init_session_context(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * bac
 
 	/* Initialize temp tables */
 	pool_temp_tables_init();
+	
+#ifdef NOT_USED
+	/* Initialize preferred main node id */
+	pool_reset_preferred_main_node_id();
+#endif
 
-	unset_query_cache_disabled();
+	/* Snapshot isolation state */
+	session_context->si_state = SI_NO_SNAPSHOT;
 
-	unset_query_cache_disabled_tx();
+	/* Transaction read only */
+	session_context->transaction_read_only = false;
+
+	dml_adaptive_init();
 }
 
 /*
@@ -181,6 +201,8 @@ pool_session_context_destroy(void)
 		if (session_context->query_context)
 			pool_query_context_destroy(session_context->query_context);
 		MemoryContextDelete(session_context->memory_context);
+
+		dml_adaptive_destroy();
 	}
 	/* XXX For now, just zap memory */
 	memset(&session_context_d, 0, sizeof(session_context_d));
@@ -512,6 +534,26 @@ dump_sent_message(char *caller, POOL_SENT_MESSAGE * m)
 					caller, m, m->kind, m->name, m->state)));
 }
 
+static void
+dml_adaptive_init(void)
+{
+	if (pool_config->disable_load_balance_on_write == DLBOW_DML_ADAPTIVE)
+	{
+		session_context->is_in_transaction = false;
+		session_context->transaction_temp_write_list = NIL;
+	}
+}
+
+static void
+dml_adaptive_destroy(void)
+{
+	if (pool_config->disable_load_balance_on_write == DLBOW_DML_ADAPTIVE && session_context)
+	{
+		if (session_context->transaction_temp_write_list != NIL)
+			list_free_deep(session_context->transaction_temp_write_list);
+	}
+}
+
 /*
  * Create a sent message.
  * kind: one of 'P':Parse, 'B':Bind or 'Q':Query(PREPARE)
@@ -701,10 +743,10 @@ void
 pool_set_writing_transaction(void)
 {
 	/*
-	 * If disable_transaction_on_write is 'off', then never turn on writing
+	 * If disable_transaction_on_write is 'off' or 'dml_adaptive', then never turn on writing
 	 * transaction flag.
 	 */
-	if (pool_config->disable_load_balance_on_write != DLBOW_OFF)
+	if (pool_config->disable_load_balance_on_write != DLBOW_OFF && pool_config->disable_load_balance_on_write != DLBOW_DML_ADAPTIVE)
 	{
 		pool_get_session_context(false)->writing_transaction = true;
 		ereport(DEBUG5,
@@ -807,7 +849,7 @@ pool_get_transaction_isolation(void)
 
 	/* No cached data is available. Ask backend. */
 
-	do_query(MASTER(session_context->backend),
+	do_query(MAIN(session_context->backend),
 			 "SELECT current_setting('transaction_isolation')", &res, MAJOR(session_context->backend));
 
 	error_context_stack = callback.previous;
@@ -906,7 +948,81 @@ pool_copy_prep_where(bool *src, bool *dest)
 {
 	memcpy(dest, src, sizeof(bool) * MAX_NUM_BACKENDS);
 }
+#ifdef NOT_USED
+/*
+ * Add to send map a PREPARED statement
+ */
+void
+pool_add_prep_where(char *name, bool *map)
+{
+	int			i;
 
+	if (!session_context)
+	{
+		ereport(ERROR,
+				(errmsg("pool_add_prep_where: session context is not initialized")));
+		return;
+	}
+
+	for (i = 0; i < POOL_MAX_PREPARED_STATEMENTS; i++)
+	{
+		if (*session_context->prep_where.name[i] == '\0')
+		{
+			strncpy(session_context->prep_where.name[i], name, POOL_MAX_PREPARED_NAME);
+			pool_copy_prep_where(map, session_context->prep_where.where_to_send[i]);
+			return;
+		}
+	}
+	ereport(ERROR,
+			(errmsg("pool_add_prep_where: no empty slot found")));
+}
+
+/*
+ * Search send map by PREPARED statement name
+ */
+bool *
+pool_get_prep_where(char *name)
+{
+	int			i;
+
+	if (!session_context)
+		ereport(ERROR,
+				(errmsg("pool_get_prep_where: session context is not initialized")));
+
+
+	for (i = 0; i < POOL_MAX_PREPARED_STATEMENTS; i++)
+	{
+		if (!strcmp(session_context->prep_where.name[i], name))
+		{
+			return session_context->prep_where.where_to_send[i];
+		}
+	}
+	return NULL;
+}
+
+/*
+ * Remove PREPARED statement by name
+ */
+void
+pool_delete_prep_where(char *name)
+{
+	int			i;
+
+	if (!session_context)
+		ereport(ERROR,
+				(errmsg("pool_delete_prep_where: session context is not initialized")));
+
+	for (i = 0; i < POOL_MAX_PREPARED_STATEMENTS; i++)
+	{
+		if (!strcmp(session_context->prep_where.name[i], name))
+		{
+			memset(&session_context->prep_where.where_to_send[i], 0, sizeof(bool) * MAX_NUM_BACKENDS);
+			*session_context->prep_where.name[i] = '\0';
+			return;
+		}
+	}
+}
+#endif							/* NOT_USED */
 /*
  * Initialize sent message list
  */
@@ -965,7 +1081,7 @@ can_query_context_destroy(POOL_QUERY_CONTEXT * qc)
 		{
 			count++;
 		}
-		next = lnext(cell);
+		next = lnext(session_context->pending_messages, cell);
 	}
 
 	if (count >= 1)
@@ -1270,7 +1386,7 @@ pool_pending_message_pull_out(void)
 
 	pool_pending_message_free_pending_message(m);
 	session_context->pending_messages =
-		list_delete_cell(session_context->pending_messages, cell, NULL);
+		list_delete_cell(session_context->pending_messages, cell);
 
 	MemoryContextSwitchTo(old_context);
 	return message;
@@ -1300,7 +1416,7 @@ pool_pending_message_get(POOL_MESSAGE_TYPE type)
 	{
 		POOL_PENDING_MESSAGE *m = (POOL_PENDING_MESSAGE *) lfirst(cell);
 
-		next = lnext(cell);
+		next = lnext(session_context->pending_messages, cell);
 
 		if (m->type == type)
 		{
@@ -1589,7 +1705,7 @@ pool_pending_message_get_message_num_by_backend_id(int backend_id)
 			}
 		}
 
-		next = lnext(cell);
+		next = lnext(session_context->pending_messages, cell);
 	}
 	return cnt;
 }
@@ -1622,7 +1738,7 @@ dump_pending_message(void)
 						message->type, message->contents_len, message->query, message->statement, message->portal,
 						message->node_ids[0], message->node_ids[1])));
 
-		next = lnext(cell);
+		next = lnext(session_context->pending_messages, cell);
 	}
 
 	ereport(DEBUG5,
@@ -1706,6 +1822,38 @@ pool_unset_suspend_reading_from_frontend(void)
 	session_context->suspend_reading_from_frontend = false;
 }
 
+#ifdef NOT_USED
+/*
+ * Set preferred "main" node id.
+ * Only used for SimpleForwardToFrontend.
+ */
+void
+pool_set_preferred_main_node_id(int node_id)
+{
+	session_context->preferred_main_node_id = node_id;
+}
+
+/*
+ * Return preferred "main" node id.
+ * Only used for SimpleForwardToFrontend.
+ */
+int
+pool_get_preferred_main_node_id(void)
+{
+	return session_context->preferred_main_node_id;
+}
+
+/*
+ * Reset preferred "main" node id.
+ * Only used for SimpleForwardToFrontend.
+ */
+void
+pool_reset_preferred_main_node_id(void)
+{
+	session_context->preferred_main_node_id = -1;
+}
+#endif
+
 /*-----------------------------------------------------------------------
  * Temporary table list management modules.
  *-----------------------------------------------------------------------
@@ -1870,7 +2018,7 @@ Retry:
 		{
 			ereport(DEBUG1,
 					(errmsg("pool_temp_tables_commit_pending: remove: %s", table->tablename)));
-			session_context->temp_tables = list_delete_ptr(session_context->temp_tables, table);
+			session_context->temp_tables = list_delete_cell(session_context->temp_tables, cell);
 			pool_temp_tables_dump();
 			goto Retry;
 		}
@@ -1907,7 +2055,7 @@ Retry:
 			ereport(DEBUG1,
 					(errmsg("pool_temp_tables_remove_pending: remove: %s", table->tablename)));
 
-			session_context->temp_tables = list_delete_ptr(session_context->temp_tables, table);
+			session_context->temp_tables = list_delete_cell(session_context->temp_tables, cell);
 			pool_temp_tables_dump();
 			goto Retry;
 		}
@@ -1934,125 +2082,5 @@ pool_temp_tables_dump(void)
 						table->tablename, table->state)));
 	}
 #endif
-}
 
-/*
- * Return true if an explicit transaction has been started by a
- * multi-statement-query
- */
-bool
-is_tx_started_by_multi_statement_query(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("is_tx_started_by_multi_statement_query: session context is not initialized")));
-
-	return session_context->is_tx_started_by_multi_statement;
-}
-
-/*
- * Remember that an explicit transaction has been started by a
- * multi-statement-query
- */
-void
-set_tx_started_by_multi_statement_query(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("set_tx_started_by_multi_statement_query: session context is not initialized")));
-
-	session_context->is_tx_started_by_multi_statement = true;
-}
-
-/*
- * Forget that an explicit transaction has been started by a
- * multi-statement-query
- */
-void
-unset_tx_started_by_multi_statement_query(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("unset_tx_started_by_multi_statement_query: session context is not initialized")));
-
-	session_context->is_tx_started_by_multi_statement = false;
-}
-
-/*
- * Set query_cache_disabled
- */
-void
-set_query_cache_disabled(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("set_query_cache_disabled: session context is not initialized")));
-
-	session_context->query_cache_disabled = true;
-}
-
-/*
- * Unset query_cache_disabled
- */
-void
-unset_query_cache_disabled(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("unset_query_cache_disabled: session context is not initialized")));
-
-	session_context->query_cache_disabled = false;
-}
-
-/*
- * Get query_cache_disabled
- */
-bool
-query_cache_disabled(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("query_cache_disabled: session context is not initialized")));
-
-	return session_context->query_cache_disabled ||
-		session_context->query_cache_disabled_tx;
-}
-
-/*
- * Set query_cache_disabled in transaction
- */
-void
-set_query_cache_disabled_tx(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("set_query_cache_disabled_tx: session context is not initialized")));
-
-	session_context->query_cache_disabled_tx = true;
-}
-
-/*
- * Unset query_cache_disabled in transaction
- */
-void
-unset_query_cache_disabled_tx(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("unset_query_cache_disabled_tx: session context is not initialized")));
-
-	session_context->query_cache_disabled_tx = false;
-}
-
-/*
- * Get query_cache_disabled in transaction
- */
-bool
-query_cache_disabled_tx(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("query_cache_disabled_tx: session context is not initialized")));
-
-	return session_context->query_cache_disabled_tx;
 }
