@@ -38,7 +38,7 @@ function install_pgpool
 
 	test -d $log || mkdir $log
         
-	make install HEALTHCHECK_DEBUG=1 WATCHDOG_DEBUG=1 -C $dir/../../ -e prefix=${PGPOOL_PATH} >& regression.log 2>&1
+	make install WATCHDOG_DEBUG=1 -C $dir/../../ -e prefix=${PGPOOL_PATH} >& regression.log 2>&1
 
 	if [ $? != 0 ];then
 	    echo "make install failed"
@@ -107,6 +107,9 @@ function export_env_vars
 	export PGBENCH_PATH=$PGBENCH_PATH
 	export PGSOCKET_DIR=$PGSOCKET_DIR
 	export PGVERSION=`$PGBIN/initdb -V|awk '{print $3}'|sed 's/\..*//'`
+	export LANG=C
+
+	export ENABLE_TEST=true
 }
 function print_info
 {
@@ -163,11 +166,11 @@ if [ "$MODE" = "install" ]; then
 
 elif [ "$MODE" = "noinstall" ]; then
 	echo not installing pgpool for the tests ...
-	if [[ -n "$PGPOOL_INSTALL_PATH" ]]; then
-		PGPOOL_PATH=$PGPOOL_INSTALL_PATH
+	if [[ -n "$PGPOOL_INSTALL_DIR" ]]; then
+		PGPOOL_PATH=$PGPOOL_INSTALL_DIR
 	fi
-        export PGPOOL_SETUP=$dir/../pgpool_setup
-        export WATCHDOG_SETUP=$dir/../watchdog_setup
+        export PGPOOL_SETUP=$PGPOOL_PATH/bin/pgpool_setup
+        export WATCHDOG_SETUP=$PGPOOL_PATH/bin/watchdog_setup
 else
 	echo $MODE : Invalid mode
 	exit -1
@@ -197,6 +200,13 @@ do
 	clean_all
 	timeout $TIMEOUT ./test.sh > $log/$i 2>&1
 	rtn=$?
+
+	check_segfault
+	if [ $? -eq 0 ];then
+		echo "fail: Segmentation fault detected" >> $log/$i
+		rtn=1
+	fi
+
 	if [ $rtn = 0 ];then
 		echo ${CGREEN}"ok."${CNORM}
 		ok=`expr $ok + 1`
